module raylib
   use iso_c_binding
   implicit none

   type, bind(c) :: vector2
      real(c_float) :: x
      real(c_float) :: y
   end type

   type, bind(c) :: vector3
      real(c_float) :: x
      real(c_float) :: y
      real(c_float) :: z
   end type

   type, bind(c) :: color
      integer(c_int8_t) :: r
      integer(c_int8_t) :: g
      integer(c_int8_t) :: b
      integer(c_int8_t) :: a
   end type

   type(color) :: PURPLE = color(200, 122, 255, 255)
   type(color) :: RAYWHITE = color(245, 245, 245, 255)

   type, bind(c) :: camera3d
      type(vector3) :: position
      type(vector3) :: target
      type(vector3) :: up
      real(c_float) :: fovy
      integer(c_int) :: projection
   end type

   integer(c_int) :: KEY_RIGHT = 262
   integer(c_int) :: KEY_LEFT = 263
   integer(c_int) :: KEY_DOWN = 264
   integer(c_int) :: KEY_UP = 265

   integer(c_int) :: CAMERA_PERSPECTIVE = 0

   interface

      subroutine init_window(width, height, title) bind(c, name="InitWindow")
         import :: c_char
         import :: c_int
         integer(c_int), intent(in), value :: width
         integer(c_int), intent(in), value :: height
         character(c_char), dimension(*), intent(in) :: title
      end subroutine

      function window_should_close() result(res) bind(c, name="WindowShouldClose")
         import :: c_bool
         logical(c_bool) :: res
      end function

      subroutine close_window() bind(c, name="CloseWindow")
      end subroutine

      subroutine clear_background(col) bind(c, name="ClearBackground")
         import :: color
         Type(color), intent(in), value :: col
      end subroutine

      subroutine begin_drawing() bind(c, name="BeginDrawing")
      end subroutine

      subroutine end_drawing() bind(c, name="EndDrawing")
      end subroutine

      subroutine begin_mode_3d(camera) bind(c, name="BeginMode3D")
         import :: camera3d
         type(camera3d), intent(in), value :: camera
      end subroutine

      subroutine end_mode_3d() bind(c, name="EndMode3D")
      end subroutine

      subroutine set_target_fps(fps) bind(c, name="SetTargetFPS")
         import :: c_int
         integer(c_int), intent(in), value :: fps
      end subroutine

      function is_key_down(key) result(res) bind(c, name="IsKeyDown")
         import :: c_int
         import :: c_bool
         logical(c_bool) :: res
         integer(c_int), intent(in), value :: key
      end function is_key_down

      subroutine draw_grid(slices, spacing) bind(c, name="DrawGrid")
         import :: c_int
         import :: c_float
         integer(c_int), intent(in), value :: slices
         real(c_float), intent(in), value :: spacing
      end subroutine draw_grid

      subroutine draw_cube(position, width, height, length, col) bind(c, name="DrawCube")
         import :: vector3
         import :: c_float
         import :: color
         type(vector3), intent(in), value :: position
         real(c_float), intent(in), value :: width
         real(c_float), intent(in), value :: height
         real(c_float), intent(in), value :: length
         type(color), intent(in), value :: col
      end subroutine draw_cube

      function get_time() result(res) bind(c, name="GetTime")
         import :: c_double
         real(c_double) :: res
      end function get_time
   end interface
end module
